######################################################################
# @@HEADER2_NANAMI@@
######################################################################
# v0.2.1 First Release
######################################################################

use strict;
use Nana::Enc;
use Nana::OpenID;

$login::imeon="ime-mode: active;";
$login::imeoff="ime-mode: disabled;";
$login::captchaon=5;

$login::loginforms1="text|userid,password|password,select|expire|login_plugin_login_expire,submit|submit|login_plugin_login_button";
$login::loginforms2="text|userid,password|password,captcha|captcha,select|expire|login_plugin_login_expire,submit|submit|login_plugin_login_button";

# readuser writeuser readwrite
$login::useraddpolicy="writeuser";

# none, days, admin
$login::useraddpolicy2="days";

$login::writewaitdays=7;

$login::formexpire=24*60*60;

$login::useraddforms="text|userid,password|pass1,password|pass2,mail|email,mailconfirm|emailchk,captcha|captcha,submit|submit|login_plugin_useradd_button";
$login::useraddforms="text|nickname,select|sex|login_plugin_useradd_sex_value,date|birthday,text|userid,password|pass1,password|pass2,mail|email,mailconfirm|emailchk,captcha|captcha,submit|submit|login_plugin_useradd_button";

$login::forgetforms="text|email,mailconfirm|emailchk,captcha|captcha,submit|submit|login_plugin_forget_button";

$login::style={
	nickname=>{
		style=>$login::imeon,
		size=>"200,15",
		length=>"4,32",
		onkeypress=>"k_nickname",,
		onchange=>"c_nickname",
		onsubmit=>"s_nickname",
	},
	userid=>{
		style=>$login::imeoff,
		size=>"100,15",
		length=>"4,16",
		check=>\&plugin_login_check_userid,
		onkeypress=>"k_userid",
		onchange=>"c_userid",
		onsubmit=>"s_userid",
	},
	password=>{
		style=>$login::imeoff,
		size=>"100,15",
		length=>"6,32",
		check=>\&plugin_login_check_password,
		onkeypress=>"k_passwd",
		onchange=>"c_passwd",
		onsubmit=>"s_passwd",
	},
	pass1=>{
		style=>$login::imeoff,
		size=>"100,15",
		length=>"6,32",
		check=>\&plugin_login_check_newpass,
		objs=>"pass1,pass2",
		onkeypress=>"k_newpass",
		onchange=>"c_newpass",
		onsubmit=>"s_newpass",
	},
	pass2=>{
		style=>$login::imeoff,
		size=>"100,15",
		length=>"6,32",
		check=>\&plugin_login_check_newpass,
		objs=>"pass1,pass2",
		onkeypress=>"k_newpass",
		onchange=>"c_newpass",
		onsubmit=>"s_newpass",
	},
	email=>{
		style=>$login::imeoff,
		size=>"200,15",
		length=>"8,256",
		check=>\&plugin_login_check_email,
		objs=>"email,emailchk",
		onkeypress=>"k_email",
		onchange=>"c_email",
		onsubmit=>"s_email",
	},
	emailchk=>{
		style=>$login::imeoff,
		size=>"100,15",
		length=>"8,256",
		check=>\&plugin_login_check_email,
		objs=>"email,emailchk",
		onkeypress=>"k_email",
		onchange=>"c_email",
		onsubmit=>"s_email",
	},
	confirm=>{
		style=>$login::imeoff,
		size=>"200,15",
		length=>"31,33",
#		check=>\&plugin_login_check_confirm,
	},
	expire=>{
		size=>"150,20",
	},
	submit=>{
		size=>"150,20",
	},
};

#$::login_enq_form="enq";
#$::login_enq_form_cols=50;

sub login_read_resource {
	my ($file, %r)=@_;
$::debug.="r:$file\n";

	if(-r "$::res_dir/$file.$::lang.txt") {
		%r=&read_resource("$::res_dir/$file.$::lang.txt", %r);
$::debug.="r:$::res_dir/$file.$::lang.txt\n";
	} elsif(-r "$::res_dir/$file.txt") {
		%r=&read_resource("$::res_dir/$file.txt", %r);
$::debug.="r:$::res_dir/$file.txt\n";
	}
	%r;
}

sub plugin_login_action {
	my $body;
	my $login_plugin_title=$::resource{login_plugin_login};

	my $mode=$::form{mode};
	if($::useExPlugin eq 1 && $::_exec_plugined{login} eq 2) {
		if($mode eq "logout") {
			return &plugin_login_logout;
		} elsif($mode eq "login" || $mode eq "") {
			return &plugin_login_login;
		} elsif($mode eq "mypage") {
			return &plugin_login_mypage;
		} elsif($mode eq "useradd") {
			return &plugin_login_useradd;
		} elsif($mode eq "openid") {
			return &plugin_login_openid;
		} elsif($mode eq "forget") {
			return &plugin_login_forget;
		} elsif($mode eq "pass") {
			return &plugin_login_pass;
		} elsif($mode eq "mail") {
			return &plugin_login_mail;
		} elsif($mode eq "confirm") {
			return &plugin_login_confirm;
		} elsif($mode eq "unregist") {
			return &plugin_login_unregist;
		} elsif($mode eq "regist") {
			return &plugin_login_regist;
		} elsif($mode eq "teamofuse") {
			return &plugin_login_teamofuse;
		}
	}
	return ('msg'=>"\tError", 'body'=>qq(<span class="error">login.inc.pl error</span>));
}

sub plugin_login_teamofuse {
	my $body;

	%::resource=&login_read_resource("login_teamofuse", %::resource);
	$::nowikiname = 1;
	&init_inline_regex;
	my $body=&replace($::resource{login_teamofuse_msg},
		TITLE=>$::wiki_title,
		script=>$::script,
		DAYS=>$login::writewaitdays,
	);

	return(msg=>"\t$::resource{login_teamofuse_title}", body=>&text_to_html($body));
}

sub form_generate {
	my($flg)=@_;
	my $body;
	my $token_form=&lf("token");
	my $expire_form=&lf("generate");

	$body.=<<EOM if($::form{refer} ne "");
<input type="hidden" name="refer" value="$::form{refer}" />
EOM
	if($flg+0) {
		my $token=$::form{$token_form};
		my $expire=$::form{$expire_form};
		if(Nana::Enc::decode("", $expire, $token) +0 < time + $login::formexpire) {
			return "-";
		}
	}
	&maketoken;
	my $token=$::Token;
	my $expire=Nana::Enc::encode(time, $token);
	return <<EOM;
$body<input type="hidden" name="$token_form" value="$token" />
<input type="hidden" name="$expire_form" value="$expire" />
EOM
}

sub plugin_login_useradd {
	my $title;
	my $body;
	my %err;

	%::resource=&login_read_resource("login_useradd", %::resource);
	$err{ok}=1;
	if($::form{&lf("back")} ne "") {
		$err{ok}=0;
	} else {
		%err=&loginchk("useradd", "$login::useraddforms","","",%err);
		$err{ok}=0;# if($err{$form} ne "");
	}
	$body.=&loginhtml("useradd", "$login::useraddforms","","",%err);

	return (msg=>"\t$::resource{login_plugin_title_msg}", body=>$body);
}


sub plugin_login_openid {
	my $body;
	if($::_module_loaded{"Net::OpenID::Consumer"}) {
		if($::form{x} eq "v") {
			my %hash=Nana::OpenID::verify();
			if($hash{status} eq "login") {
				$body.=qq(<a href="$hash{url}">$hash{url}</a><br />);
				$body.=qq(<a href="?@{[&encode($::form{refer})]}">$::form{refer}</a><br />);
			} elsif($hash{status} eq "redirect") {
				&location($hash{url}, 302, $::HTTP_HEADER);
				exit;
			} elsif($hash{status} eq "cancel") {
				my($url, $name)=Nana::OpenID::getid($::form{&lf("service")});
				my $res=$::resource{login_plugin_openid_cancel};
				$res=~s/\$1/$name/g;
				$body.=&loginhtml("login", "$login::loginforms1", $res, &plugin_login_openidform);
			} else {
				my($url, $id, $name)=Nana::OpenID::getid($::form{&lf("service")});
				my $res=$::resource{login_plugin_openid_error};
				$res=~s/\$1/$name/g;
				$res=~s/\$2/$hash{code}/g;
				$res=~s/\$3/$hash{text}/g;
				$body.=&loginhtml("login", "$login::loginforms1", $res, &plugin_login_openidform);


#	foreach(keys %hash) {
#		$body.=qq(hash $_=$hash{$_}<br />);
#	}
#	foreach(keys %::form) {
#		$body.=qq(form $_=$::form{$_}<br />);
#	}
			}
		} else {
			eval {
				my $url=Nana::OpenID::login($::form{&lf("url")}, "refer=$::form{refer}");
#$body=$url;
				&location($url, 302, $::HTTP_HEADER);
				exit;
			};
			if($@) {
				my($url, $id, $name)=Nana::OpenID::getid($::form{&lf("service")});
				my $res=$::resource{login_plugin_openid_autherr};
				$res=~s/\$1/$name/g;
				$body.=&loginhtml("login", "$login::loginforms1", $res, &plugin_login_openidform);
			}
		}
	}
	return(msg=>"\ttest", body=>$body);
}

sub plugin_login_openidform {
	my $openid;
	if($::_module_loaded{"Net::OpenID::Consumer"}) {
		foreach my $openidservice (Nana::OpenID::list()) {
			my ($url, $id, $name, $imageid, $plusid, $urlid)=Nana::OpenID::getid($openidservice, "small");
			my $refer=&encode(&encode(exists($::form{refer}) ? $::form{refer} : $::FrontPage));
			if($plusid+0 eq 0) {
				$openid.=<<EOM;
<a href="javascript:void(0);" class="openidimage" id="$imageid" onclick="return openidlogin('$::script','$refer', '@{[&lf("url")]}','@{[&encode($url)]}','@{[&lf("service")]}','$id');"><span>$name</span></a>
EOM
			} elsif($plusid+0 eq 2) {
				my $userid="userid";
				$openid.=<<EOM;
<a href="javascript:void(0);" class="openidimage" id="$imageid" onclick="return openidlogin('$::script','$refer','@{[&lf("url")]}','@{[&encode($url)]}','@{[&lf("service")]}','$id','','','login','@{[&lf("userid")]}');"><span>$name</span></a>
EOM
			} else {
				my $userid="userid";
				$openid.=<<EOM;
<a href="javascript:void(0);" class="openidimage" id="$imageid" onclick="return openidlogin('$::script','$refer','@{[&lf("url")]}','@{[&encode($url)]}','@{[&lf("service")]}','$id','login','@{[&lf("userid")]}');"><span>$name</span></a>
EOM
			}
		}
	}
	$openid;
}

sub plugin_login_login {
	my $body;

	%::resource=&login_read_resource("login_useradd", %::resource);

	$body.=&loginhtml("login", "$login::loginforms1", "", &plugin_login_openidform);
	$body.=&loginhtml("useradd", "$login::useraddforms");
	$body.=&loginhtml("forget", "$login::forgetforms");
	return(msg=>"\t$::resource{login_plugin_title_msg}", body=>$body);
}

# HTML

my $plugin_login_jsadd=0;

sub loginhtml {
	my ($title, $forms, $status, $add, %err)=@_;
	my $body;

	$body=<<EOM if($status ne "");
<span class="error">$status</span>
EOM

	$body.=<<EOM;
<h3>$::resource{"login_plugin\_$title\_title"}</h3>
<form action="$::script" method="post" name="$title">
@{[&form_generate]}
<input type="hidden" name="cmd" value="login" />
<input type="hidden" name="mode" value="$title" />
<table>
EOM
	foreach(split(/,/,$forms)) {
		my($fmode, $form, $res)=split(/\|/,$_);
		my $tmp=&loginform($fmode, $title, $form, $res, %err);
		$body.=$tmp;
	}
	my $msg=$::resource{"login_plugin\_$title\_msg"};
	$msg.=$::resource{"login_plugin\_$title\_msg\_$login::useraddpolicy"};

	$msg=&replace($msg,
		TITLE=>$::wiki_title,
		script=>$::script,
		DAYS=>$login::writewaitdays,
		login_plugin_useradd_msg=>$::resource{"login_plugin\_$title\_msg\_$login::useraddpolicy2"}
	);
	$body.=<<EOM;
<tr><td>&nbsp;</td><td colspan="2">$msg</td></tr>
EOM
	$body.=<<EOM if($add ne "");
<tr><td>&nbsp;</td><td>$add</td></tr>
EOM

	$body.=<<EOM;
</table>
</form>
EOM
	$body;
}

sub loginchk {
	my ($title, $forms, $status, $add, %err)=@_;

	foreach(split(/,/,$forms)) {
		my($fmode, $form, $res)=split(/\|/,$_);
		%err=&loginformchk($fmode, $title, $form, $res, %err);
	}
	%err;
}

sub loginform {
	my($fmode, $title, $form, $res, %err)=@_;
	my $body;
	my $value;
	my($width, $height)=split(/,/,$login::style->{$form}->{size});
	my($minlength, $maxlength)=split(/,/,$login::style->{$form}->{length});
	my $style=$login::style->{$form}->{style};

	$style.=qq( width:@{[$width]}px;) if($width+0 > 0);
	$style.=qq( height:@{[$height]}px;) if($height+0 > 0);

	$value=$value ne "" ? $value : $::form{&lf($form)};

	$value="" if($err{$form} ne "");
	if($fmode eq "text" || $fmode eq "mail") {
		if($err{ok}) {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			$body.=$value;
			$body.=qq(<input type="hidden" name="@{[&lf($form)]}" value="$value" />);
		} else {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			$body.=qq(<input type="text" name="@{[&lf($form)]}" maxlength="$maxlength" value="$value"@{[$style eq "" ? "" : qq( style="$style")]} />\n); 
		}

	} elsif($fmode eq "password") {
		if($err{ok}) {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			my $passwd_form;
			eval {
				$passwd_form=&passwordform($value, "hidden", &lf($form), $::form{"$login::passform\_enc"}, $::form{"login::passform\_token"}, $maxlength, $maxlength, $style);
			};
			$body.=$passwd_form;
		} else {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			my $passwd_form;
			eval {
				$passwd_form=&passwordform($value, "", &lf($form), $::form{"$login::passform\_enc"}, $::form{"login::passform\_token"}, $maxlength, $maxlength, $style);
			};
			$body.=$passwd_form;
		}

	} elsif($fmode eq "date") {
		if($err{ok}) {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			$body.=<<EOM;
$::form{&lf($form . "y")}$::resource{login_plugin_useradd_date_year}
$::form{&lf($form . "m")}$::resource{login_plugin_useradd_date_mon}
$::form{&lf($form . "d")}$::resource{login_plugin_useradd_date_day}
<input type="hidden" name="@{[&lf($form . "y")]}" value="$::form{&lf($form . "y")}" />
<input type="hidden" name="@{[&lf($form . "m")]}" value="$::form{&lf($form . "m")}" />
<input type="hidden" name="@{[&lf($form . "d")]}" value="$::form{&lf($form . "d")}" />
EOM

		} else {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
			$value=$::form{&lf($form . "y")};
			$body.=qq(<select name="@{[&lf($form . "y")]}"@{[$style eq "" ? "" : qq( style="$style")]}>);
			my @years;
			for(my $i=1900; $i<=&date("Y"); $i++) {
				my $s="$i$::resource{login_plugin_useradd_date_year}";
				if($::lang eq "ja") {
					foreach(split(/,/,$::resource{login_plugin_useradd_date_wareki})) {
						my($start, $end, $str)=split(/\|/,$_);
						if($i >= $start && $i <= $end) {
							my $wa=$i-$start+1;
							$wa=$::resource{login_plugin_useradd_date_gannen} if($wa+0 <= 1);
							push(@years, "$i\t$s ($str$wa$::resource{login_plugin_useradd_date_year})");
						}
					}
				} else {
					push(@years, "$i\t$s");
				}
			}
			foreach(reverse @years) {
				my($y, $s)=split(/\t/, $_);
				$body.=qq(<option value="$y"@{[$y eq $value ? ' selected="selected"' : '']}>$s</option>);
			}
			$body.=qq(</select>);
			$value=$::form{&lf($form . "m")};
			$body.=qq(<select name="@{[&lf($form . "m")]}"@{[$style eq "" ? "" : qq( style="$style")]}>);
			for(my $i=1; $i<=12; $i++) {
				$body.=qq(<option value="$i"@{[$i eq $value ? ' selected="selected"' : '']}>$i$::resource{login_plugin_useradd_date_mon}</option>);
			}
			$body.=qq(</select>);
			$value=$::form{&lf($form . "d")};
			$body.=qq(<select name="@{[&lf($form . "d")]}"@{[$style eq "" ? "" : qq( style="$style")]}>);
			for(my $i=1; $i<=31; $i++) {
				$body.=qq(<option value="$i"@{[$i eq $value ? ' selected="selected"' : '']}>$i$::resource{login_plugin_useradd_date_day}</option>);
			}
			$body.=qq(</select>);
		}

	} elsif($fmode eq "mailconfirm") {
		if($err{ok}+0 eq 0) {
			if(    $err{$form} ne ""
				|| $err{$form . "1"} ne ""
				|| $err{$form . "2"} ne "") {
				$::form{&lf($form . "1")}="";
				$::form{&lf($form . "2")}="";
			}
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
#			$body.=qq(<input type="hidden" name="@{[&lf($form)]}" value="@{[$::form{&lf($form)}]}" />);
			$body.=qq(<input type="text" name="@{[&lf($form . "1")]}" maxlength="$maxlength" value="@{[$::form{&lf($form . "1")}]}"@{[$style eq "" ? "" : qq( style="$style")]} />\n); 
			$body.='@';
			$body.=qq(<input type="text" name="@{[&lf($form . "2")]}" maxlength="$maxlength" value="@{[$::form{&lf($form . "2")}]}"@{[$style eq "" ? "" : qq( style="$style")]} />\n); 
		}
	} elsif($fmode eq "captcha") {
		if($err{ok}+0 eq 0) {
			$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td colspan="2">);
			my $captcha_form;
			eval {
				$captcha_form=&plugin_captcha_form;
			};
			$body=$captcha_form eq "" ? "" : $body . $captcha_form;
		}
	} elsif($fmode eq "select") {
		my $sel=$::resource{$res} eq "" ? $res : $::resource{$res};
		$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
		if($err{ok}) {
			foreach(split(/,/,$sel)) {
				my($n, $v)=split(/\|/,$_);
				if($n eq $value) {
					$body.=$v;
				}
			}
			$body.=qq(<input type="hidden" name="@{[&lf($form)]}" value="$value" />);
		} else {
			$body.=qq(<select name="@{[&lf($form)]}"@{[$style eq "" ? "" : qq( style="$style")]}>);
			foreach(split(/,/,$sel)) {
				my($n, $v)=split(/\|/,$_);
				if($n eq $value) {
					$body.=qq(<option value="$n" selected="selected">$v</option>);
				} else {
					$body.=qq(<option value="$n">$v</option>);
				}
			}
			$body.=qq(</select>\n);
		}
	} elsif($fmode eq "radio") {
	} elsif($fmode eq "submit") {
		$body.=qq(<tr><td>$::resource{"login_plugin\_$title\_$form\_title"}</td><td>);
		my $sel=$::resource{"$res\_back"} eq "" ? $res : $::resource{"$res\_back"};
		$body.=qq(<input type="submit" name="@{[&lf("back")]}" value="$sel" />\n)
			if($err{ok});
		$sel=$::resource{$res} eq "" ? $res : $::resource{$res};
		$body.=qq(<input type="submit" name="@{[&lf($form)]}" value="$sel" />\n);
	}
	if($err{$form} ne "") {
		$body.=qq(<td><span class="error">$err{$form}</span></td>);
	} else {
		$body.=qq(<td>&nbsp;</td>);
	}
	return $body eq "" ? "" : "$body</tr>\n";
}

sub loginformchk {
	my($fmode, $title, $form, $res, %err)=@_;
	my($minlength, $maxlength)=split(/,/,$login::style->{$form}->{length});
	my $value=$::form{&lf($form)};
	my $chkmethod=undef;
	if(defined($login::style->{$form}->{check})) {
		$chkmethod=$login::style->{$form}->{check};
	}

	if($fmode eq "text" || $fmode eq "mail" || $fmode eq "password") {
		if($minlength+0 > 0) {
			if(length($value) eq 0) {
				$err{$form}=$::resource{"login_plugin_useradd_$form\_noinput"};
				$err{ok}=0;
			}
		}
		if($err{$form} eq "" && $minlength+0 > 0 && $maxlength+0 > 0) {
			if(length($value) < $minlength || length($value) > $maxlength) {
				$err{$form}=$::resource{"login_plugin_useradd_$form\_length"};
				$err{$form}=~s/\$MIN/$minlength/g;
				$err{$form}=~s/\$MAX/$maxlength/g;
				$err{ok}=0;
			}
		}
	}
	if($chkmethod ne undef) {
		%err=&$chkmethod($form, $value, %err);
	}
	%err;
}

sub plugin_login_check_userid {
	my($f, $s, %err)=@_;
	%err;
}
sub plugin_login_check_password {
	my($f, $s, %err)=@_;
	%err;
}
sub plugin_login_check_newpass {
	my($f, $s, %err)=@_;
	return %err if($err{$f} ne "");
	if($::form{&lf("pass1")} ne $::form{&lf("pass2")}) {
		$err{$f}=$::resource{"login_plugin_useradd_$f\_err"};
	} elsif($s=~/[\,\=\\\$\'\"]/) {
		$err{$f}=$::resource{"login_plugin_useradd_$f\_ignorestr"};
	} else {
		my $flg=0;
		$flg++ if($s=~/[A-Z]/);
		$flg++ if($s=~/[a-z]/);
		$flg++ if($s=~/[0-9]/);
		if($flg < 3) {
			$err{$f}=$::resource{"login_plugin_useradd_$f\_char"};
		}
	}
	%err;
}
sub plugin_login_check_email {
	my($f, $s, %err)=@_;
	return %err if($err{"email"} ne "");

	%::resource=&login_read_resource("login_mailaddr", %::resource);

	if($::form{&lf("email")} ne 
		$::form{&lf("emailchk1")} . '@' . $::form{&lf("emailchk2")}) {
		$err{$f}=$::resource{"login_plugin_useradd_email_diff"};
	}
	if($::form{&lf("email")}!~/$::ismail/o) {
		$err{$f}=$::resource{"login_plugin_useradd_email_err"};
	}
	my @mx;
	if(&load_module("Net::DNS")) {
		@mx=Net::DNS::mx($::form{&lf("emailchk2")});
		$err{$f}=$::resource{"login_plugin_useradd_email_notdomain"}
			if($#mx < 0);
	}

	foreach("mobile", "sp", "msnlive", "freemail") {
		%err=&chkmaildomain($_, $f, $::form{&lf("emailchk2")}, %err);
	}

	%err;
}

sub chkmaildomain {
	my($m, $f, $domain, %err)=@_;
	my $list=$::resource{"login_$m"};
	my $flg=0;
$::debug.="check $m of $domain\n";
	my @mx;
	if(&load_module("Net::DNS")) {
		@mx=Net::DNS::mx($::form{&lf("emailchk2")});
	}

	foreach my $l(split(/\n/,$list)) {
		$flg=1 if($domain=~/$l/);
		$flg=1 if($l=~/$domain/);
		foreach my $mx(@mx) {
			my $m=$mx->exchange;
			$flg=1 if($l=~/$m/);
			$flg=1 if($m=~/$l/);
		}
	}
	$err{$f}=$::resource{"login_plugin_useradd_email_$m\_match_pc"} if($flg);
#	$err{$f}=$::resource{"login_plugin_useradd_email_$m\_match_sp"} if($flg);
#	$err{$f}=$::resource{"login_plugin_useradd_email_$m\_match_mobile"} if($flg);
	%err;
}

1;
